/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package com.example.company.ejb;

import java.rmi.RemoteException;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.ejb.Stateless;
import javax.jms.BytesMessage;
import javax.jms.Destination;
import javax.jms.JMSException;
import javax.jms.Message;
import javax.jms.MessageConsumer;
import javax.jms.QueueConnection;
import javax.jms.QueueConnectionFactory;
import javax.jms.Session;
import javax.jms.TextMessage;
import javax.naming.InitialContext;
import javax.naming.NamingException;

/**
 *
 * @author slossr1
 */
@Stateless
public class StatelessSessionConsumerEjbBean implements StatelessSessionConsumerEjbBeanRemote {
    
    //input queue jndi name, must be defined in Glassfish
    private String INPUT_Q = "jndi/INPUT.Q";

    //xa enabled jms queue connection factory jndi name
    private String conFactory = "jms/XA_Connection_Factory";
    
    //queue connection, session, consumer and jndi context are initialised later
    private QueueConnection connection;
	private Session session;
	private MessageConsumer messageConsumer;
	private InitialContext jndiContext;

    /**
     * Initialises required jndiContext, connection factory, session, consumer and
     * starts the connection ready to retrieve messages.
     *
     * @throws javax.naming.NamingException
     * @throws javax.jms.JMSException
     */
    private void init() throws NamingException, JMSException {
        jndiContext = new InitialContext();
        QueueConnectionFactory connectionFactory =
			(QueueConnectionFactory)jndiContext.lookup(conFactory);
        connection = connectionFactory.createQueueConnection();
        session = connection.createSession(false, 0);
        Destination destination = (Destination)jndiContext.lookup(INPUT_Q);
        messageConsumer = session.createConsumer(destination);
        connection.start();
    }

    /**
     * Consumes a single message from the input queue and outputs content to console.
     *
     * @return boolean indicating if consumed a message
     * @throws RemoteException
     */
    public boolean consumeFromQueue() throws RemoteException {
        try {
            init();
            Message message;
            message = messageConsumer.receive(1000);
            String text;
            if(message == null)
                return false;
            if (message instanceof TextMessage)
            {
                text = ((TextMessage) message).getText();
            }
            else
            {
                byte[] body = new byte[(int) ((BytesMessage) message).getBodyLength()];
                ((BytesMessage) message).readBytes(body);
                text = new String(body);
            }
            Logger.getLogger(StatelessSessionConsumerEjbBean.class.getName()).log(Level.INFO,
                    "consumed message: "+text);
            closeResources();
            return true;
        } catch (NamingException ex) {
            Logger.getLogger(StatelessSessionConsumerEjbBean.class.getName()).log(Level.SEVERE,
                    "NamingException encountered whilst trying to consume from the input queue", ex);
            throw new RemoteException(ex.getMessage());
        } catch (JMSException ex) {
            Logger.getLogger(StatelessSessionConsumerEjbBean.class.getName()).log(Level.SEVERE,
                    "JMSException encountered whilst trying to consume from the input queue", ex);
            throw new RemoteException(ex.getMessage());
        }
    }

    /**
     * Closes open resources.
     *
     * @throws javax.jms.JMSException
     * @throws javax.naming.NamingException
     */
    private void closeResources() throws JMSException, NamingException {
		if(messageConsumer!=null)
			messageConsumer.close();
        if(session!=null)
            session.close();
        if(connection!=null)
            connection.close();
        if(jndiContext!=null)
            jndiContext.close();
    }
}
